/*
    Visma Compact 5 API

    Copyright (C) 2005-2010, Visma Spcs AB
    
    $Id: XorBase5.h,v 1.40 2010/10/18 07:46:49 VSW\Dahn_Nil Exp $
 */

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>

#ifdef XORBASE5_EXPORTS
#define XORBASE5_API __declspec(dllexport)
#else
#define XORBASE5_API __declspec(dllimport)
#endif

/* Constants for reserved accounts                              */
#define DEFVATOUT           1       /* Sales VAT                */
#define DEFVATIN            10      /* Purchase VAT             */
#define DEFSTOCK            18      /* Stock                    */
#define DEFYEARRESULT       19      /* Year Result (balance)    */
#define DEFCUSTOMER         20      /* Accounts receivable      */
#define DEFFREIGHT          21      /* Charged freights         */
#define DEFFEE              22      /* Invoice fees             */
#define DEFADJUSTMENT       23      /* Rounding                 */
#define DEFINTERIN          24      /* Interest income          */
#define DEFFINANCE          25      /* Financial entries        */
#define DEFSALES            26      /* Sales                    */
#define DEFPURCHASE         27      /* Purchase                 */
#define DEFSUPPLIER         28      /* Accounts payable         */
#define DEFPAYIN            29      /* Accounts receivable (bank)*/
#define DEFCASH             30      /* Cash                     */
#define DEFINTEROUT         31      /* Interest expenses        */
#define DEFCHANGE           32      /* Adjustments              */
#define DEFDISCOUNTOUT      33      /* Discount out             */
#define DEFDISCOUNTIN       34      /* Discount in              */
#define DEFXPROFIT          35      /* Exchange income          */
#define DEFXLOSS            36      /* Exchange loss            */
#define DEFEXPSALES         37      /* Export EU sales          */
#define DEFEXPSALES2        38      /* Other Export sales       */
#define DEFPAYOUT           39      /* Accounts payable (bank ) */
#define DEFVAT              40      /* VAT settle account       */
#define DEFFREIGHTEXP       41      /* Export freight account   */
#define DEFFEEEXP           42      /* Export fee account       */
#define DEFYEARRESULTBOOK   43      /* Year Result (result)     */
#define DEFFREIGHTEU        44      /* Export freight account   */
#define DEFFEEEU            45      /* Export fee account       */
#define DEFREVVAT           46      /* Default account for reversed VAT */
#define DEFACCRUEDRECEIVABLE 47     /* Default account for accrued receivable */
#define DEFACCRUEDDEBT      48      /* Default account for accrued debt */

/* Constants for person types */
#define CUSTOM          1
#define SUPPLY          2

/* Constants for number types */
#define XORDER          0
#define XCUSTINVOICE    1
#define XPURCHASE       2
#define XSUPPINVOICE    3
#define XPAY            4

/* Constants for invoice status */
#define NOTPRINTED      0
#define PRINTED         1
#define TRANSFERED      2
#define PERMANENT       3
#define NOTREADY        4

/* Constants for ledger types */
#define FA              0
#define KN              1
#define RF              2
#define AC              3
#define BE              4

/* Table Definitions */
#define CLIENT                          0
#define ACCOUNT                         1
#define OBJECT                          2
#define BALANCE                         3
#define VOUCHER                         4
#define AUTO                            5
#define DEFAULT                         6
#define TEXTREG                         7
#define HISTORY                         8
#define PERSON                          9
#define PERSSALES                       10
#define LEDGER                          11
#define INVOICE                         12
#define INVOICEROW                      13
#define ARTICLE                         14
#define ARTSALES                        15
#define ACCYEAR                         16
#define SERIES                          17
#define RELATION                        18
#define ACCCHART                        19
#define SETTING                         20
#define PERSSETTINGS                    21
#define DOCLOG                          22
#define ACCRUAL                         23

/* Database return codes (errors) */
#define DbError     -1  /* general error                                */
#define DbOk        0   /* OK                                           */
#define DbNotFound  1   /* record not found                             */
#define DbEof       2   /* end of file                                  */
#define DbBof       3   /* beginning of file                            */
#define DbDuplicate 4   /* primary key already exists                   */


#ifdef __cplusplus
extern "C" {
#endif

/* XORBASE5 Specific functions */

typedef PVOID Field;
typedef PVOID Key;
typedef unsigned short Date;

#pragma pack(4)

// Client structure for list
typedef struct
{
    char id[16];
    char name[48];
} ClientName;

#pragma pack()


// Open a Visma Compact database. The path to the database is
// read from the registry. If DatabaseSetPath has been called that
// path is used instead.
XORBASE5_API int WINAPI DatabaseOpen( LPCSTR clientNumber );

// Close the database. Releases all resources allocated.
XORBASE5_API void WINAPI DatabaseClose();

// Set the path to the database. May be used if XOR Compact has not been installed.
XORBASE5_API void WINAPI DatabaseSetPath( LPCSTR path );

// Returns the path to current database.
XORBASE5_API int WINAPI DatabaseGetPath( LPSTR path, int maxPathSize );

// Returns the number of tables in the current database.
XORBASE5_API int WINAPI DatabaseTableCount();

XORBASE5_API void WINAPI SetErrorHandler( int (CALLBACK *ErrorCallback)(int, int, char *) );

// Returns the number of fields in this table.
XORBASE5_API int WINAPI TableFieldCount( int tableIndex);

// Returns the number of records in this table.
XORBASE5_API int WINAPI TableRecordCount( int tableIndex);

// Returns the name of this table.
XORBASE5_API int WINAPI TableName( int tableIndex, LPSTR buffer, int bufferSize );

// Create a new blank record for this table.
XORBASE5_API int WINAPI RecordCreate( int tableIndex);

// Find a specific record in this table
XORBASE5_API int WINAPI RecordFind( Key key);

// Move the current record to the first record in this table with the sort key specified
XORBASE5_API int WINAPI RecordFirst( int tableIndex, int keyNumber);

// Move the current record to next position in this table
XORBASE5_API int WINAPI RecordNext( int tableIndex);

// Move the current record to previous position in this table
XORBASE5_API int WINAPI RecordPrev( int tableIndex);

// Move the current record to last position in this table
XORBASE5_API int WINAPI RecordLast( int tableIndex, int keyNumber);

// Write the current record to the table
XORBASE5_API int WINAPI RecordUpdate( int tableIndex);

// Add the current record to the table
XORBASE5_API int WINAPI RecordAdd( int tableIndex);

// Deletes the current record
XORBASE5_API int WINAPI RecordDelete( int tableIndex);

// Return a fieldhandle from this table
XORBASE5_API Field WINAPI RecordField( int tableIndex, LPCSTR fieldName);

XORBASE5_API Key WINAPI RecordKey( int tableIndex, int keyNumber);

XORBASE5_API Field WINAPI KeyField( Key key, LPCSTR fieldName);

// Returns the length of the given field
XORBASE5_API int WINAPI FieldLength( Field field);

// Returns the type of the field
XORBASE5_API int WINAPI FieldType( Field field);

// Returns the name of the field
XORBASE5_API int WINAPI FieldName( int tableIndex, int fieldIndex, LPSTR buffer, int bufferSize);

XORBASE5_API int WINAPI FieldAssignDouble( Field field, double value);
XORBASE5_API int WINAPI FieldAssignLong( Field field, int value);
XORBASE5_API int WINAPI FieldAssignString( Field field, LPCSTR string);
XORBASE5_API int WINAPI FieldAssignBool( Field field, bool value);
XORBASE5_API int WINAPI FieldAssignDate( Field field, Date value);

XORBASE5_API double WINAPI FieldValueDouble( Field field);
XORBASE5_API int WINAPI FieldValueLong( Field field);
XORBASE5_API int WINAPI FieldValueString( Field field, LPSTR buffer, int bufferSize);
XORBASE5_API bool WINAPI FieldValueBool( Field field);
XORBASE5_API Date WINAPI FieldValueDate( Field field);

XORBASE5_API int WINAPI InvoiceFind(int serieNumber, int invoiceIndex, int invoiceNumber);
XORBASE5_API int WINAPI InvoiceRowFind(int serieNumber, int invoiceIndex, int invoiceNumber, int rowNumber);

// Return yearNo if successful
XORBASE5_API int WINAPI InvoiceOpen( LPCSTR personID, int type, int status, int year, int month, int day, double freight, double fee);
// Returns transaction row number if successful
XORBASE5_API int WINAPI InvoiceAddRow( LPCSTR articleID, LPCSTR articleName, LPCSTR accountNumber, LPCSTR object1, LPCSTR object2, LPCSTR unit, double items, double price, double discount);
// Returns the invoice number if successful
XORBASE5_API int WINAPI InvoiceClose();

XORBASE5_API int WINAPI VoucherFind(int yearNumber, int series, int voucherNumber, int rowNumber);

// Return yearNo if successful
XORBASE5_API int WINAPI VoucherOpen( int series, int year, int month, int day);
// Returns transaction row number if successful
XORBASE5_API int WINAPI VoucherAddRow( LPCSTR accountNumber, LPCSTR voucherText, LPCSTR obj1, LPCSTR obj2, double amount );
// Returns the voucher number if successful
XORBASE5_API int WINAPI VoucherClose();

XORBASE5_API int WINAPI GetReservedAccount( int defaultNumber, LPSTR accountNumber, int size);
XORBASE5_API int WINAPI GetStandardText( int textNumber, LPSTR buffer, int bufferSize);

XORBASE5_API int WINAPI PersonFind(int personType, LPCSTR personID);
XORBASE5_API int WINAPI PersonAdd( int personType, LPCSTR personID, LPCSTR personName, LPCSTR group, LPCSTR adress1, LPCSTR adress2, LPCSTR adress3, LPCSTR adress4, LPCSTR phoneNumber, LPCSTR faxNumber, LPCSTR reference, int termsOfPayment, int termsOfDelivery, int wayOfDelivery);

XORBASE5_API int WINAPI ArticleFind( LPCSTR articleNumber);
XORBASE5_API int WINAPI ArticleAdd( LPCSTR articleNumber, LPCSTR name, LPCSTR group, double itemsStock, double minItems, double price, double purchasePrice);

XORBASE5_API int WINAPI ObjectFind(LPCSTR objID);
XORBASE5_API int WINAPI ObjectAdd( int objectType, LPCSTR objectID, LPCSTR name, int spanYear);

XORBASE5_API int WINAPI LedgerFind(int serieNumber, int personType, int ledgerType, int ledgerNumber);

XORBASE5_API int WINAPI LedgerAddCustomerInvoice( LPCSTR personID, int year, int month, int day, double amount, double vat, int ledType, int invoiceNumber, int update, LPCSTR account);
XORBASE5_API int WINAPI LedgerAddSupplierInvoice( LPCSTR personID, LPCSTR suppliersInvoiceNo, int year, int month, int day, double amount, double vat, int ledType, int update, LPCSTR account);

XORBASE5_API WORD WINAPI DueDateCalc( LPCSTR buf, WORD begin);

/* Date functions */
XORBASE5_API WORD WINAPI DateSystem(void);
XORBASE5_API WORD WINAPI YMDToDate(int year, int mon, int day, int check);
XORBASE5_API WORD WINAPI FirstDayOfWeek( int year);
XORBASE5_API WORD WINAPI DayOfWeek( WORD julianDate );
XORBASE5_API WORD WINAPI BeginWeek( WORD julianDate );
XORBASE5_API WORD WINAPI EndWeek( WORD julianDate );
XORBASE5_API WORD WINAPI EndWorkWeek( WORD julianDate );
XORBASE5_API WORD WINAPI DayOfYear( WORD julianDate );
XORBASE5_API WORD WINAPI BeginYear( WORD julianDate );
XORBASE5_API WORD WINAPI EndYear( WORD julianDate );
XORBASE5_API WORD WINAPI BeginMon( WORD julianDate );
XORBASE5_API WORD WINAPI EndMon( WORD julianDate );
XORBASE5_API int WINAPI Year( WORD julianDate );
XORBASE5_API int WINAPI Month( WORD julianDate );
XORBASE5_API int WINAPI Day( WORD julianDate );
XORBASE5_API int WINAPI Week( WORD julianDate );

XORBASE5_API int WINAPI GetYearNo( WORD julianDate);
XORBASE5_API int WINAPI IsDateLocked( WORD julianDate);

XORBASE5_API int WINAPI ExportSieFile( LPCSTR fileName, int level, WORD julianDate);
XORBASE5_API int WINAPI ImportSieFile( LPCSTR fileName);

XORBASE5_API int WINAPI ClientListInit(void *buffer);
XORBASE5_API int WINAPI ClientListNext(void *buffer);

XORBASE5_API void WINAPI DocumentAddSupplierInvoice( LPCSTR filePath, int storeDocument);

#ifdef __cplusplus
}
#endif
