{
    XOR Compact 5 API

    Copyright (C) 1996-2010, Visma Spcs AB
    
    $Id: XorBase5.pas,v 1.17 2010/10/18 13:08:14 VSW\Dahn_Nil Exp $
}

unit Xorbase5;

interface

uses
    Windows;

{ Constants for reserved accounts }
const DEFVATOUT         = 1;  { Sales VAT }
const DEFVATIN          = 10; { Purchase VAT }
const DEFSTOCK          = 18; { Stock }
const DEFYEARRESULT     = 19; { Year results }
const DEFCUSTOMER       = 20; { Accounts receivable }
const DEFFREIGHT        = 21; { Charged freights }
const DEFFEE            = 22; { Invoice fees }
const DEFADJUSTMENT     = 23; { Even-Out }
const DEFINTERIN        = 24; { Interest income }
const DEFFINANCE        = 25; { Financial entries }
const DEFSALES          = 26; { Sales }
const DEFPURCHASE       = 27; { Purchase }
const DEFSUPPLIER       = 28; { Accounts payable }
const DEFPAYIN          = 29; { Accounts receivable (bank) }
const DEFCASH           = 30; { Cash }
const DEFINTEROUT       = 31; { Interest expenses }
const DEFCHANGE         = 32; { Adjustments }
const DEFDISCOUNTOUT    = 33; { Discount out }
const DEFDISCOUNTIN     = 34; { Discount in }
const DEFXPROFIT        = 35; { Exchange income }
const DEFXLOSS          = 36; { Exchange loss }
const DEFEXPSALES       = 37; { Export EU sales }
const DEFEXPSALES2      = 38; { Other Export sales }
const DEFPAYOUT         = 39; { Accounts payable (bank ) }
const DEFVAT            = 40; { VAT settle account }
const DEFFREIGHTEXP     = 41; { Export freight account }
const DEFFEEEXP         = 42; { Export fee account }
const DEFYEARRESULTBOOK = 43; { Year Result (result) }
const DEFFREIGHTEU      = 44; { Export freight account }
const DEFFEEEU          = 45; { Export fee account }
const DEFREVVAT         = 46; { Default account for reversed VAT }
const DEFACCRUEDRECEIVABLE = 47; { Default account for accrued receivable }
const DEFACCRUEDDEBT    = 48; { Default account for accrued debt }

{ Constants for person types }
const CUSTOM = 1;
const SUPPLY = 2;

{ Constants for number types }
const XORDER        = 0;
const XCUSTINVOICE  = 1;
const XPURCHASE     = 2;
const XSUPPINVOICE  = 3;
const XPAY          = 4;

{ Constants for invoice status }
const NOTPRINTED = 0;
const PRINTED    = 1;
const TRANSFERED = 2;
const PERMANENT  = 3;
const NOTREADY   = 4;

{ Constants for ledger types }
const FA = 0;
const KN = 1;
const RF = 2;
const AC = 3;
const BE = 4;

{ Table Definitions }
const XCLIENT     = 0;
const XACCOUNT    = 1;
const XOBJECT     = 2;
const XBALANCE    = 3;
const XVOUCHER    = 4;
const XAUTO       = 5;
const XDEFAULT    = 6;
const XTEXT       = 7;
const XHISTORY    = 8;
const XPERSON     = 9;
const XPERSSALES  = 10;
const XLEDGER     = 11;
const XINVOICE    = 12;
const XINVOICEROW = 13;
const XARTICLE    = 14;
const XARTSALES   = 15;
const XXYEAR      = 16;
const XSERIES     = 17;
const XRELATION   = 18;
const XACCCHART   = 19;
const XSETTING    = 20;
const XPERSSETTINGS = 21;
const XDOCLOG     = 22;
const XACCRUAL    = 23;

{ Database return codes (errors) }
const DbError    = -1; { general error }
const DbOk 	       = 0;  { OK }
const DbNotFound  = 1;  { record not found }
const DbEof       = 2;  { end of file }
const DbBof       = 3;  { beginning of file }
const DbDuplicate = 4;  { primary key already exists }


function DatabaseOpen( clientNumber: PChar ): Integer stdcall;

procedure DatabaseClose() stdcall;

function DatabasePack( clientNumber: PChar ): Integer stdcall;

procedure DatabaseSetPath( path: PChar) stdcall;

function DatabaseGetPath( path: PChar;  maxPathSize: Integer ): Integer stdcall;

function DatabaseTableCount(): Integer stdcall;

procedure SetErrorHandler( ErrorCallback: Pointer ) stdcall;

function TableFieldCount( tableIndex: Integer): Integer stdcall;

function TableRecordCount( tableIndex: Integer): Integer stdcall;

function TableName( tableIndex, buffer: PChar; bufferSize: Integer ): Integer stdcall;

function RecordCreate( tableIndex: Integer): Integer stdcall;

function RecordFind( key: Pointer): Integer stdcall;

function RecordFirst( tableIndex, keyNumber: Integer): Integer stdcall;

function RecordNext( tableIndex: Integer): Integer stdcall;

function RecordPrev( tableIndex: Integer): Integer stdcall;

function RecordLast( tableIndex, keyNumber: Integer): Integer stdcall;

function RecordUpdate( tableIndex: Integer): Integer stdcall;

function RecordAdd( tableIndex: Integer): Integer stdcall;

function RecordDelete( tableIndex: Integer): Integer stdcall;

function RecordField( tableIndex: Integer; fieldName: PChar): Pointer stdcall;

function FieldLength( field: Pointer): Integer stdcall;

function FieldName( tableIndex, fieldIndex: Integer; buffer: PChar; bufferSize: Integer): Integer stdcall;

function FieldType( field: Pointer): Integer stdcall;

function FieldAssignDouble( field: Pointer; value: Double): Integer stdcall;
function FieldAssignLong( field: Pointer; value: Integer): Integer stdcall;
function FieldAssignString( field: Pointer; stringVal: PChar): Integer stdcall;
function FieldAssignBool( field: Pointer; value: Boolean): Integer stdcall;
function FieldAssignDate( field: Pointer; value: Word): Integer stdcall;

function FieldValueDouble( field: Pointer): double stdcall;
function FieldValueLong( field: Pointer): Integer stdcall;
function FieldValueString( field: Pointer; buffer: PChar; bufferSize: Integer): Integer stdcall;
function FieldValueBool( field: Pointer): Boolean stdcall;
function FieldValueDate( field: Pointer): Word stdcall;

function RecordKey( tableIndex, keyNumber: Integer): Pointer stdcall;
function KeyField( key: Pointer; fieldName: PChar): Pointer stdcall;

function InvoiceFind( serieNumber, invoiceIndex, invoiceNumber: Integer): Integer stdcall;
function InvoiceRowFind( serieNumber, invoiceIndex, invoiceNumber, rowNumber: Integer): Integer stdcall;

function InvoiceOpen( persNumber: PChar; invType, status, year, month, day: Integer; freight, fee: Double): Integer stdcall;
function InvoiceAddRow( articleID, articleName, accountNumber, object1, object2, unitText: PChar; items, price, discount: Double): Integer stdcall;
function InvoiceClose(): Integer stdcall;

function VoucherFind( yearNumber, series, voucherNumber, rowNumber: Integer): Integer stdcall;

function VoucherOpen( series, year, month, day: Integer): Integer stdcall;
function VoucherAddRow( accountNumber, voucherText, obj1, obj2: PChar; amount: Double): Integer stdcall;
function VoucherClose(): Integer stdcall;

function LedgerFind ( serieNumber, personType, ledgerType, ledgerNumber: Integer):Integer stdcall;

function LedgerAddCustomerInvoice(personNumber: PChar; year, month, day: Integer; amount, vat: Double; ledType, invoiceNumber, update: Integer; account: PChar): Integer stdcall;
function LedgerAddSupplierInvoice(personNumber, suppliersInvoiceNo: PChar; year, month, day: Integer; amount, vat: Double; ledType, update: Integer; account: PChar): Integer stdcall;
procedure DocumentAddSupplierInvoice(filePath: PChar; storeDocument: Integer) stdcall;

function GetReservedAccount( defaultNumber: Integer; accountNumber: PChar; size: Integer): Integer stdcall;
function GetStandardText( textNumber: Integer; buffer: PChar; bufferSize: Integer): Integer stdcall;
function ObjectFind( objNo: PChar ): Integer stdcall;
function ObjectAdd( objectType: Integer; objNo, name: PChar; spanYear: Integer): Integer stdcall;

function PersonFind( persType: Integer; personID: PChar): Integer stdcall;

function PersonAdd( persType: Integer; persNo, name, group, adr1, adr2, adr3, adr4, phone, fax, ref: PChar; top, tod, wod: Integer): Integer stdcall;
function ArticleFind( artNo: PChar ): Integer stdcall;
function ArticleAdd( artNo, name, group: PChar; itemsStock, minItems, price, purchasePrice: Double): Integer stdcall;

function DueDateCalc( buf: PChar; beginDate: Word): Word stdcall;

{ constants for BumpDate function }
const XYEAR      = 1;
const XMON       = 2;
const XDAY       = 3;
const XBEGINYEAR = 4;
const XENDYEAR   = 5;
const XBEGINMON  = 6;
const XENDMON    = 7;
const XLEAPYEAR  = 8;
const XDAYOFYEAR = 9;
const XWEEK      = 10;

function DateSystem: Word stdcall;

function YMDToDate(year: Integer;
                   mon: Integer;
                   day: Integer;
                   check: Integer): Word stdcall;


{ Julian date functions }
function FirstDayOfWeek(y: Word): Word;
function DayOfWeek(jul: Word): Word;
function BeginWeek(jul: Word): Word;
function EndWeek(jul: Word): Word;
function EndWorkWeek(jul: Word): Word;
function DayOfYear(jul: Word): Word;
function BeginYear(jul: Word): Word;
function EndYear(jul: Word): Word;
function BeginMon(jul: Word): Word;
function EndMon(jul: Word): Word;
function Year(jul: Word): Word;
function Mon(jul: Word): Word;
function Day(jul: Word): Word;
function Week(jul: Word): Word;
function GetYearNo(jul: Word): Integer stdcall;
function IsDateLocked(jul: Word): Integer stdcall;

{ Miscellaneous functions }
type ClientNameType = record
  id: array[0..16-1] of Char;
  clname: array[0..48-1] of Char;
end {ClientNameType};

function ExportSieFile( fileName: PChar; level: Integer; julianDate: Word): Integer stdcall;
function ImportSieFile( fileName: PChar): Integer stdcall;
function ClientListInit ( clientName: Pointer): Integer stdcall;
function ClientListNext ( clientName: Pointer): Integer stdcall;

{ ------ Functions implemented in this unit ------- }

{ Convert a Julian date to a Delphi date. }
function JulianToDelphi(x: Word): TDateTime;
{ Convert a Delphi date to a Julian date. }
function DelphiToJulian(x: TDateTime): Word;

implementation

function DatabaseOpen( clientNumber: PChar ): Integer stdcall; external 'xorbase5.dll';

procedure DatabaseClose() stdcall; external 'xorbase5.dll';

function DatabasePack( clientNumber: PChar ): Integer stdcall; external 'xorbase5.dll';

procedure DatabaseSetPath( path: PChar) stdcall; external 'xorbase5.dll';

function DatabaseGetPath( path: PChar;  maxPathSize: Integer ): Integer stdcall; external 'xorbase5.dll';

function DatabaseTableCount(): Integer stdcall; external 'xorbase5.dll';

procedure SetErrorHandler( ErrorCallback: Pointer ) stdcall; external 'xorbase5.dll';

function TableFieldCount( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function TableRecordCount( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function TableName( tableIndex, buffer: PChar; bufferSize: Integer ): Integer stdcall; external 'xorbase5.dll';

function RecordCreate( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordFind( key: Pointer): Integer stdcall; external 'xorbase5.dll';

function RecordFirst( tableIndex, keyNumber: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordNext( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordPrev( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordLast( tableIndex, keyNumber: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordUpdate( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordAdd( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordDelete( tableIndex: Integer): Integer stdcall; external 'xorbase5.dll';

function RecordField( tableIndex: Integer; fieldName: PChar): Pointer stdcall; external 'xorbase5.dll';

function FieldLength( field: Pointer): Integer stdcall; external 'xorbase5.dll';

function FieldName( tableIndex, fieldIndex: Integer; buffer: PChar; bufferSize: Integer): Integer stdcall; external 'xorbase5.dll';

function FieldType( field: Pointer): Integer stdcall; external 'xorbase5.dll';

function FieldAssignDouble( field: Pointer; value: Double): Integer stdcall; external 'xorbase5.dll';
function FieldAssignLong( field: Pointer; value: Integer): Integer stdcall; external 'xorbase5.dll';
function FieldAssignString( field: Pointer; stringVal: PChar): Integer stdcall; external 'xorbase5.dll';
function FieldAssignBool( field: Pointer; value: Boolean): Integer stdcall; external 'xorbase5.dll';
function FieldAssignDate( field: Pointer; value: Word): Integer stdcall; external 'xorbase5.dll';

function FieldValueDouble( field: Pointer): double stdcall; external 'xorbase5.dll';
function FieldValueLong( field: Pointer): Integer stdcall; external 'xorbase5.dll';
function FieldValueString( field: Pointer; buffer: PChar; bufferSize: Integer): Integer stdcall; external 'xorbase5.dll';
function FieldValueBool( field: Pointer): Boolean stdcall; external 'xorbase5.dll';
function FieldValueDate( field: Pointer): Word stdcall; external 'xorbase5.dll';

function RecordKey( tableIndex, keyNumber: Integer): Pointer stdcall; external 'xorbase5.dll';
function KeyField( key: Pointer; fieldName: PChar): Pointer stdcall; external 'xorbase5.dll';

function InvoiceFind( serieNumber, invoiceIndex, invoiceNumber: Integer): Integer stdcall; external 'xorbase5.dll';
function InvoiceRowFind( serieNumber, invoiceIndex, invoiceNumber, rowNumber: Integer): Integer stdcall; external 'xorbase5.dll';

function InvoiceOpen( persNumber: PChar; invType, status, year, month, day: Integer; freight, fee: Double): Integer stdcall; external 'xorbase5.dll';
function InvoiceAddRow( articleID, articleName, accountNumber, object1, object2, unitText: PChar; items, price, discount: Double): Integer stdcall;  external 'xorbase5.dll';
function InvoiceClose(): Integer stdcall; external 'xorbase5.dll';

function VoucherFind( yearNumber, series, voucherNumber, rowNumber: Integer): Integer stdcall; external 'xorbase5.dll';

function VoucherOpen( series, year, month, day: Integer): Integer stdcall; external 'xorbase5.dll';
function VoucherAddRow( accountNumber, voucherText, obj1, obj2: PChar; amount: Double): Integer stdcall; external 'xorbase5.dll';
function VoucherClose(): Integer stdcall; external 'xorbase5.dll';

function LedgerFind (serieNumber, personType, ledgerType, ledgerNumber: Integer):Integer stdcall; external 'xorbase5.dll';

function LedgerAddCustomerInvoice(personNumber: PChar; year, month, day: Integer; amount, vat: Double; ledType, invoiceNumber, update: Integer; account: PChar): Integer stdcall; external 'xorbase5.dll';
function LedgerAddSupplierInvoice(personNumber, suppliersInvoiceNo: PChar; year, month, day: Integer; amount, vat: Double; ledType, update: Integer; account: PChar): Integer stdcall; external 'xorbase5.dll';
procedure DocumentAddSupplierInvoice(filePath: PChar; storeDocument: Integer) stdcall; external 'xorbase5.dll';

function GetReservedAccount( defaultNumber: Integer; accountNumber: PChar; size: Integer): Integer stdcall; external 'xorbase5.dll';
function GetStandardText( textNumber: Integer; buffer: PChar; bufferSize: Integer): Integer stdcall; external 'xorbase5.dll';
function ObjectFind( objNo: PChar ): Integer stdcall; external 'xorbase5.dll';
function ObjectAdd( objectType: Integer; objNo, name: PChar; spanYear: Integer): Integer stdcall; external 'xorbase5.dll';
function PersonFind( persType: Integer; personID: PChar): Integer stdcall; external 'xorbase5.dll';
function PersonAdd( persType: Integer; persNo, name, group, adr1, adr2, adr3, adr4, phone, fax, ref: PChar; top, tod, wod: Integer): Integer stdcall; external 'xorbase5.dll';
function ArticleFind( artNo: PChar ): Integer stdcall; external 'xorbase5.dll';
function ArticleAdd( artNo, name, group: PChar; itemsStock, minItems, price, purchasePrice: Double): Integer stdcall; external 'xorbase5.dll';

function DueDateCalc( buf: PChar; beginDate: Word): Word stdcall; external 'xorbase5.dll';

function DateSystem: Word stdcall; external 'xorbase5.dll';

function YMDToDate(year: Integer;
                   mon: Integer;
                   day: Integer;
                   check: Integer): Word stdcall; external 'xorbase5.dll';


function FirstDayOfWeek(y: Word): Word; external 'xorbase5.dll';
function DayOfWeek(jul: Word): Word; external 'xorbase5.dll';
function BeginWeek(jul: Word): Word; external 'xorbase5.dll';
function EndWeek(jul: Word): Word; external 'xorbase5.dll';
function EndWorkWeek(jul: Word): Word; external 'xorbase5.dll';
function DayOfYear(jul: Word): Word; external 'xorbase5.dll';
function BeginYear(jul: Word): Word; external 'xorbase5.dll';
function EndYear(jul: Word): Word; external 'xorbase5.dll';
function BeginMon(jul: Word): Word; external 'xorbase5.dll';
function EndMon(jul: Word): Word; external 'xorbase5.dll';
function Year(jul: Word): Word; external 'xorbase5.dll';
function Mon(jul: Word): Word; external 'xorbase5.dll';
function Day(jul: Word): Word; external 'xorbase5.dll';
function Week(jul: Word): Word; external 'xorbase5.dll';
function GetYearNo(jul: Word): Integer stdcall; external 'xorbase5.dll';
function IsDateLocked(jul: Word): Integer stdcall; external 'xorbase5.dll';

function ExportSieFile( fileName: PChar; level: Integer; julianDate: Word): Integer stdcall; external 'xorbase5.dll';
function ImportSieFile( fileName: PChar): Integer stdcall; external 'xorbase5.dll';
function ClientListInit ( clientName: Pointer): Integer stdcall; external 'xorbase5.dll';
function ClientListNext ( clientName: Pointer): Integer stdcall; external 'xorbase5.dll';

const
    DATE1980 = 29220.00;

function JulianToDelphi(x: Word): TDateTime;
begin
    Result := DATE1980 + x;
end;

function DelphiToJulian(x: TDateTime): Word;
begin
    Result := Trunc(x - DATE1980);
end;

end.
