Attribute VB_Name = "Xorbase5"
'
'   XOR Compact 5 API
'
'    Copyright (C) 1996-2010, Visma Spcs AB
'
'   $Id: XORBase5.bas,v 1.18 2010/10/19 09:12:03 VSW\Dahn_Nil Exp $
'
' Import module for interfacing Visual Basic 6 applications to XORBase5.dll
'

'Module XORBase5

' Database return codes (errors)
    Global Const DbError = -1       ' General error
    Global Const DbOk = 0           ' OK, no error
    Global Const DbNotFound = 1     ' Record not found
    Global Const DbEof = 2          ' End of file
    Global Const DbBof = 3          ' Beginning of file
    Global Const DbDuplicate = 4    ' Primary key already exists

' Constants for person types
    Global Const CUSTOM = 1
    Global Const SUPPLY = 2

' Constants for number types
    Global Const XORDER = 0
    Global Const XCUSTINVOICE = 1
    Global Const XPURCHASE = 2
    Global Const XSUPPINVOICE = 3
    Global Const XPAY = 4

' Constants for invoice status
    Global Const NOTPRINTED = 0
    Global Const PRINTED = 1
    Global Const TRANSFERED = 2
    Global Const PERMANENT = 3
    Global Const NOTREADY = 4

' Constants for ledger types
    Global Const FA = 0
    Global Const KN = 1
    Global Const RF = 2
    Global Const AC = 3
    Global Const BE = 4

' Constants for XORBASE tables
    Global Const CLIENT = 0
    Global Const ACCOUNT = 1
    Global Const ACCOUNTINGOBJECT = 2   ' Used to be OBJECT
    Global Const BALANCE = 3
    Global Const VOUCHER = 4
    Global Const AUTOCALC = 5           ' Used to be AUTO
    Global Const DEFAULTACCOUNT = 6     ' Used to be DEFAULT
    Global Const TEXT = 7
    Global Const HISTORY = 8
    Global Const PERSON = 9
    Global Const PERSSALES = 10
    Global Const LEDGER = 11
    Global Const INVOICE = 12
    Global Const INVOICEROW = 13
    Global Const ARTICLE = 14
    Global Const ARTSALES = 15
    Global Const ACCYEAR = 16
    Global Const SERIES = 17
    Global Const RELATION = 18
    Global Const ACCCHART = 19
    Global Const SETTING = 20
    Global Const PERSSETTINGS = 21
    Global Const DOCLOG = 22
    Global Const ACCRUAL = 23

'Constants for default (reserved) accounts
    Global Const DEFVATOUT = 1          ' Sales VAT
    Global Const DEFVATIN = 10          ' Purchase VAT
    Global Const DEFSTOCK = 18          ' Stock
    Global Const DEFYEARRESULT = 19     ' Year results
    Global Const DEFCUSTOMER = 20       ' Accounts receivable
    Global Const DEFFREIGHT = 21        ' Charged freights
    Global Const DEFFEE = 22            ' Invoice fees
    Global Const DEFADJUSTMENT = 23     ' Rounding
    Global Const DEFINTERIN = 24        ' Interest income
    Global Const DEFFINANCE = 25        ' Financial entries
    Global Const DEFSALES = 26          ' Sales
    Global Const DEFPURCHASE = 27       ' Purchase
    Global Const DEFSUPPLIER = 28       ' Accounts payable
    Global Const DEFPAYIN = 29          ' Accounts receivable (bank)
    Global Const DEFCASH = 30           ' Cash
    Global Const DEFINTEROUT = 31       ' Interest expenses
    Global Const DEFCHANGE = 32         ' Adjustments
    Global Const DEFDISCOUNTOUT = 33    ' Discount out
    Global Const DEFDISCOUNTIN = 34     ' Discount in
    Global Const DEFXPROFIT = 35        ' Exchange income
    Global Const DEFXLOSS = 36          ' Exchange loss
    Global Const DEFEXPSALES = 37       ' Export sales
    Global Const DEFEXPSALES2 = 38      ' Import purchases
    Global Const DEFPAYOUT = 39         ' Accounts payable (bank )
    Global Const DEFVAT = 40            ' VAT settle account
    Global Const DEFFREIGHTEXP = 41     ' Export freight account
    Global Const DEFFEEEXP = 42         ' Export fee account
    Global Const DEFYEARRESULTBOOK = 43 ' Year Result (result)
    Global Const DEFFREIGHTEU = 44      ' Export freight account
    Global Const DEFFEEEU = 45          ' Export fee account
    Global Const DEFREVVAT = 46         ' Default account for reversed VAT
    Global Const DEFACCRUEDRECEIVABLE = 47 ' Default account for accrued receivable
    Global Const DEFACCRUEDDEBT = 48    ' Default account for accrued debt
  
  
' --------------------
' API functions
' --------------------
    Declare Function DatabaseOpen Lib "XORBASE5.DLL" (ByVal ClientNumber As String) As Long
    Declare Sub DatabaseClose Lib "XORBASE5.DLL" ()
    Declare Sub DatabaseSetPath Lib "XORBASE5.DLL" (ByVal Path As String)
    Declare Function DatabaseGetPath Lib "XORBASE5.DLL" (ByVal Path As String, ByVal StringSize As Long) As Long
    Declare Function DatabaseTableCount Lib "XORBASE5.DLL" () As Long
    Declare Function SetErrorHandler Lib "XORBASE5.DLL" (ByVal ErrorCallback As Long) As Integer

    Declare Function TableFieldCount Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function TableRecordCount Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function TableName Lib "XORBASE5.DLL" (ByVal Path As String, ByVal StringSize As Long) As Long

    Declare Function RecordCreate Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordFind Lib "XORBASE5.DLL" (ByVal Key As Any) As Long
    Declare Function RecordFirst Lib "XORBASE5.DLL" (ByVal TableIndex As Long, ByVal KeyNumber As Long) As Long
    Declare Function RecordNext Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordPrev Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordLast Lib "XORBASE5.DLL" (ByVal TableIndex As Long, ByVal KeyNumber As Long) As Long
    Declare Function RecordUpdate Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordAdd Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordDelete Lib "XORBASE5.DLL" (ByVal TableIndex As Long) As Long
    Declare Function RecordField Lib "XORBASE5.DLL" (ByVal TableIndex As Long, ByVal FieldName As String) As Long
    Declare Function RecordKey Lib "XORBASE5.DLL" (ByVal TableIndex As Long, ByVal KeyNumber As Long) As Long

    Declare Function FieldLength Lib "XORBASE5.DLL" (ByVal Field As Any) As Long
    Declare Function FieldType Lib "XORBASE5.DLL" (ByVal Field As Any) As Long
    Declare Function FieldName Lib "XORBASE5.DLL" (ByVal TableIndex As Long, ByVal FieldIndex As Long, ByVal StringBuffer As String, ByVal StringSize As Long) As Long

    Declare Function FieldAssignDouble Lib "XORBASE5.DLL" (ByVal Field As Any, ByVal value As Double) As Long
    Declare Function FieldAssignLong Lib "XORBASE5.DLL" (ByVal Field As Any, ByVal value As Long) As Long
    Declare Function FieldAssignString Lib "XORBASE5.DLL" (ByVal Field As Any, ByVal value As String) As Long
    Declare Function FieldAssignBool Lib "XORBASE5.DLL" (ByVal Field As Any, ByVal value As Boolean) As Long
    Declare Function FieldAssignDate Lib "XORBASE5.DLL" (ByVal Field As Any, ByVal value As Integer) As Long

    Declare Function FieldValueDouble Lib "XORBASE5.DLL" (ByVal Field As Any) As Double
    Declare Function FieldValueLong Lib "XORBASE5.DLL" (ByVal Field As Any) As Long
    Declare Function FieldValueBool Lib "XORBASE5.DLL" (ByVal Field As Any) As Boolean
    Declare Function FieldValueDate Lib "XORBASE5.DLL" (ByVal Field As Any) As Integer
    Declare Function FieldValueString Lib "XORBASE5.DLL" (ByVal Field As Long, ByVal StringBuffer As String, ByVal StringSize As Long) As Long
    Declare Function KeyField Lib "XORBASE5.DLL" (ByVal Key As Any, ByVal FieldName As String) As Long

    Declare Function InvoiceFind Lib "XORBASE5.DLL" (ByVal serieNumber As Long, ByVal invoiceIndex As Long, ByVal invoiceNumber As Long) As Long
    Declare Function InvoiceRowFind Lib "XORBASE5.DLL" (ByVal serieNumber As Long, ByVal invoiceIndex As Long, ByVal invoiceNumber As Long, ByVal rowNumber As Long) As Long

    Declare Function InvoiceOpen Lib "XORBASE5.DLL" (ByVal PersNumber As String, ByVal persType As Long, ByVal Status As Long, ByVal Year As Long, ByVal Month As Long, ByVal Day As Long, ByVal Freight As Double, ByVal Fee As Double) As Long
    Declare Function InvoiceAddRow Lib "XORBASE5.DLL" (ByVal ArticleID As String, ByVal ArticleName As String, ByVal AccountNumber As String, ByVal Object1 As String, ByVal Object2 As String, ByVal Unit As String, ByVal Items As Double, ByVal Price As Double, ByVal Discount As Double) As Long
    Declare Function InvoiceClose Lib "XORBASE5.DLL" () As Long

    Declare Function VoucherFind Lib "XORBASE5.DLL" (ByVal yearNumber As Long, ByVal series As Long, ByVal voucherNumber As Long, ByVal rowNumber As Long) As Long

    Declare Function VoucherOpen Lib "XORBASE5.DLL" (ByVal Series As Long, ByVal Year As Long, ByVal Month As Long, ByVal Day As Long) As Long
    Declare Function VoucherAddRow Lib "XORBASE5.DLL" (ByVal AccountNumber As String, ByVal VoucherText As String, ByVal Object1 As String, ByVal Object2 As String, ByVal amount As Double) As Long
    Declare Function VoucherClose Lib "XORBASE5.DLL" () As Long

    Declare Function LedgerFind Lib "XORBASE5.DLL" (ByVal serieNumber As Long, ByVal personType As Long, ByVal ledgerType As Long, ByVal ledgerNumber As Long) As Long

    Declare Function LedgerAddCustomerInvoice Lib "XORBASE5.DLL" (ByVal PersNumber As String, ByVal year as Long, ByVal month as Long, ByVal day as Long, ByVal amount as Double, ByVal vat as Double, ByVal ledType as Long, ByVal invoiceNumber As Long, ByVal update as Long, Byval account as String) As Long
    Declare Function LedgerAddSupplierInvoice Lib "XORBASE5.DLL" (ByVal PersNumber As String, ByVal suppliersInvoiceNumber As String, ByVal year as Long, ByVal month as Long, ByVal day as Long, ByVal amount as Double, ByVal vat as Double, ByVal ledType as Long, ByVal update as Long, Byval account as String) As Long
    Declare Sub DocumentAddSupplierInvoice Lib "XORBASE5.DLL" (ByVal filePath As String, ByVal storeDocument As Long)

    Declare Function GetReservedAccount Lib "XORBASE5.DLL" (ByVal DefaultNumber As Long, ByVal AccountNumber As String, ByVal StringSize As Long) As Long
    Declare Function GetStandardText Lib "XORBASE5.DLL" (ByVal TextNumber As Long, ByVal TextString As String, ByVal StringSize As Long) As Long

    Declare Function PersonFind Lib "XORBASE5.DLL" (ByVal personType As Long, ByVal personID As String) As Long


    ' Add a person
    Declare Function PersonAdd Lib "XORBASE5.DLL" ( _
        ByVal PersonType As Long, _
        ByVal PersonNumber As String, _
        ByVal PersonName As String, _
        ByVal Group As String, _
        ByVal Address1 As String, _
        ByVal Address2 As String, _
        ByVal Address3 As String, _
        ByVal Address4 As String, _
        ByVal PhoneNumber As String, _
        ByVal FaxNumber As String, _
        ByVal Reference As String, _
        ByVal TermsOfPayment As Long, _
        ByVal TermsOfDelivery As Long, _
        ByVal WayOfDelivery As Long) As Long

    Declare Function ArticleFind Lib "XORBASE5.DLL" ( ByVal ArticleNumber As String ) As Long

    ' Add an article
    Declare Function ArticleAdd Lib "XORBASE5.DLL" ( _
        ByVal ArticleNumber As String, _
        ByVal ArticleName As String, _
        ByVal Group As String, _
        ByVal ItemsInStock As Double, _
        ByVal OrderPoint As Double, _
        ByVal Price As Double, _
        ByVal PurchasePrice As Double) As Long

    ' Find an object
    Declare Function ObjectFind Lib "XORBASE5.DLL" ( ByVal ObjectID As String ) As Long

    ' Add an object
    Declare Function ObjectAdd Lib "XORBASE5.DLL" ( _
        ByVal objectType As Long, _
        ByVal ObjectID As String, _
        ByVal Name As String, _
        ByVal SpanYear As Long) As Long


    Declare Function DueDateCalc Lib "XORBASE5.DLL" (ByVal TermsOfPayment As String, ByVal StartDate As Integer) As Integer

' -----------------------------------
' Date functions
' -----------------------------------
Declare Function DateSystem Lib "XORBASE5.DLL" () As Long
Declare Function YMDToDate Lib "XORBASE5.DLL" (ByVal Year As Long, ByVal mon As Long, ByVal Day As Long, ByVal check As Long) As Integer
Declare Function FirstDayOfWeek Lib "XORBASE5.DLL" (ByVal Year As Integer) As Integer
Declare Function DayOfWeek Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function BeginWeek Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function EndWeek Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function EndWorkWeek Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function DayOfYear Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function BeginYear Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function EndYear Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function BeginMon Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function EndMon Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function Year Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function Month Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function Day Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function Week Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Integer
Declare Function GetYearNo Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Long
Declare Function IsDateLocked Lib "XORBASE5.DLL" (ByVal julianDate As Integer) As Long

' -----------------------------------
' Miscellaneous functions
' -----------------------------------
Declare Function ExportSieFile Lib "XORBASE5.DLL" (ByVal FileName As String, ByVal Level As Long, ByVal julianDate As Integer) As Long
Declare Function ImportSieFile Lib "XORBASE5.DLL" (ByVal FileName As String) As Long

Type ClientNameType
  id As String * 16
  name As String * 48
End Type

Declare Function ClientListInit Lib "XORBASE5.DLL" ( clientName As Any) As Integer
Declare Function ClientListNext Lib "XORBASE5.DLL" ( clientName As Any) As Integer

